/* $Id: basedlg.cpp 17 2006-12-15 03:39:43Z mawerick $ */
/* AOCrayon - Chat color management tool for Anarchy Online
    Copyright (C) 2005 - 2006  Mawerick

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
#include "aocrayon.h"
#include "basedlg.h"
#include "advmode_dlg.h"
#include "basicmode_dlg.h"
#include "settings_dlg.h"
#include "about_dlg.h"
#include "license_dlg.h"

#include <QTabWidget>
#include <QStackedWidget>
#include <QPushButton>
#include <QImage>
#include <QPixmap>
#include <QLayout>
#include <QColorDialog>
#include <QDir>
#include <QFile>
#include <QDomNode>
#include <QTextStream>

#include "AOcrayon.xpm"

/*
 *  Constructs a BaseDlg as a child of 'parent'
 */
BaseDlg::BaseDlg( QWidget* parent )
	: QDialog( parent )
{
	setWindowFlags( Qt::Dialog | Qt::WindowTitleHint | Qt::WindowSystemMenuHint );
	setWindowTitle( PROGRAM " v" VERSION );
	setWindowIcon( QPixmap( AOcrayon_xpm ) );

	setFixedSize( 360, 456 );

	BaseDlgLayout = new QVBoxLayout( this );
	BaseDlgLayout->setSpacing( 4 );
	BaseDlgLayout->setMargin( 6 );

	TabWidget = new QTabWidget( this );
//	TabWidget->setMargin( 3 );

	/*
	 * Widgetstack for keeping basic/advanced mode color editing
	 */
	ColorsTab = new QStackedWidget;
	TabWidget->addTab( ColorsTab, "Chat Colors" );

	/*
	 * Chat color list (basic mode)
	 */
	BasicModeTab = new BasicModeDlg( ColorsTab );
	connect( BasicModeTab, SIGNAL( pickColor( const QStringList& ) ), this, SLOT( pickColor( const QStringList& ) ) );
	ColorsTab->addWidget( BasicModeTab );

	/*
	 * Chat color list (advanced mode)
	 */
	AdvModeTab = new AdvancedModeDlg( ColorsTab );
	connect( AdvModeTab, SIGNAL( pickColor( const QString& ) ), this, SLOT( pickColor( const QString& ) ) );
	ColorsTab->addWidget( AdvModeTab );

	/*
	 * Settings
	 */
	SettingsTab = new SettingsDlg;
	connect( SettingsTab, SIGNAL( pickColor( const QString& ) ), this, SLOT( pickColor( const QString& ) ) );
	connect( SettingsTab, SIGNAL( reloadData() ), this, SLOT( loadData() ) );
	connect( SettingsTab, SIGNAL( modeChanged( bool ) ), this, SLOT( modeChanged( bool ) ) );
	TabWidget->addTab( SettingsTab, "Settings" );

	/*
	 * About text
	 */
	AboutTab = new AboutDlg;
	TabWidget->addTab( AboutTab, "About..." );

	/*
	 * License text
	 */
	LicenseTab = new LicenseDlg;
	TabWidget->addTab( LicenseTab, "License" );

	BaseDlgLayout->addWidget( TabWidget );

	/*
	 * Save and close buttons
	 */
	SysLayout = new QHBoxLayout;
	SysLayout->setSpacing( 4 );
	SysLayout->setMargin( 0 );

	SaveBtn = new QPushButton( this );
	SaveBtn->setText( "&Save" );
	SaveBtn->setDefault( TRUE );
	connect( SaveBtn, SIGNAL( clicked() ), this, SLOT( saveData() ) );
	SysLayout->addWidget( SaveBtn );

	QSpacerItem* spacer_2 = new QSpacerItem( 20, 20, QSizePolicy::Expanding, QSizePolicy::Minimum );
	SysLayout->addItem( spacer_2 );

	CloseBtn = new QPushButton( this );
	CloseBtn->setText( "&Close" );
	connect( CloseBtn, SIGNAL( clicked() ), this, SLOT( accept() ) );
	SysLayout->addWidget( CloseBtn );

	BaseDlgLayout->addLayout( SysLayout );

	/* load preferences, save return value for backup check */
	int ret = loadPrefs();

	/* confirm the AO path. if the user clicks cancel, close the program */
	if( SettingsTab->confirmAOPath() == FALSE )
		exit( -1 );

	/* if we failed to load prefs, we assume this is the first time the app is used
	 * and force a backup of what we believe is the original file */
	if( ret == FALSE )
		SettingsTab->backupOriginalData();

	/* load color data */
	loadData();
}

/*
 *  Destroys the object and frees any allocated resources
 */
BaseDlg::~BaseDlg()
{
	// no need to delete child widgets, Qt does it all for us
}

bool BaseDlg::loadPrefs()
{
	/* set default values first */
	AdvMode = FALSE;
	BGColor.setName( PROGRAM " background" );
	BGColor.setValue( "black" );
	updateBGColor();

	/* load the config file */
	QFile CfgFile( PROGRAM ".cfg" );
	qDebug( "loadPrefs(): file: %s", qPrintable( CfgFile.fileName() ) );
	if( !CfgFile.open( QIODevice::ReadOnly | QIODevice::Text ) )
		return FALSE;

	QTextStream Config( &CfgFile );
	QString Buffer;
	while( !Config.atEnd() )
	{
		Buffer = Config.readLine();
		qDebug( "loadPrefs(): line: %s", qPrintable( Buffer ) );
		QStringList List( Buffer.split( " = " ) );
		if( !List[0].compare( "AODir" ) )
		{
			qDebug( "loadPrefs(): setting AOPath to %s", qPrintable( List[1] ) );
			SettingsTab->setAOPath( List[1] );
		}
		if( !List[0].compare( "AdvancedMode" ) )
		{
			qDebug( "loadPrefs(): setting Advanced mode to %d", List[1].toInt() );
			SettingsTab->setMode( List[1].toInt() );
		}
		if( !List[0].compare( "BGColor" ) )
		{
			qDebug( "loadPrefs():setting BGColor to %s", qPrintable( List[1] ) );
			BGColor.setColor( List[1] );
			updateBGColor();
		}
	}
	CfgFile.close();
	return TRUE;
}

void BaseDlg::savePrefs()
{
	QFile CfgFile( PROGRAM ".cfg" );
	if( CfgFile.open( QIODevice::WriteOnly | QIODevice::Text ) )
	{
		QTextStream Config( &CfgFile );
		Config << "## " PROGRAM " Configuration ##\n";
		Config << "AODir = " << SettingsTab->getAOPath() << "\n";
		Config << "AdvancedMode = " << AdvMode << "\n";
		Config << "BGColor = " << BGColor.value() << "\n";
		CfgFile.close();
	}
}

void BaseDlg::loadData()
{
	QDomDocument xmlDoc;
	Colors.clear();
	QFile TextColors( SettingsTab->getAOPath() + GUIPATH + COLORFILE );

	qDebug( "loadData(): file: %s", qPrintable( TextColors.fileName() ) );

	if( TextColors.open( QIODevice::ReadOnly ) )
	{
		if( xmlDoc.setContent( &TextColors ) )
		{
			QDomNodeList nodeList = xmlDoc.elementsByTagName( "HTMLColor" );
			for( int i = 0; i < (int) nodeList.count(); i++ )
			{
				AOColor color;
				QDomNode node = nodeList.item( i );
				QDomElement element = node.toElement();
				color.setValue( element.attribute( "color" ) );
				color.setName( element.attribute( "name" ) );
				Colors.append( color );
			}
		}
		TextColors.close();
	}
	updateColors();
}

void BaseDlg::saveData()
{
	QDir GUIDir( SettingsTab->getAOPath() + GUIPATH );
	QFile XmlFile( GUIDir.filePath( COLORFILE ) );
	GUIDir.remove( COLORFILE ".bak" );
	GUIDir.rename( COLORFILE, COLORFILE ".bak" );
	if( XmlFile.open( QIODevice::WriteOnly ) )
	{
		QDomDocument doc( "" );
		QDomElement root = doc.createElement( "Root" );
		doc.appendChild( root );
		for( AOColors::iterator it = Colors.begin(); it != Colors.end(); it++ )
		{
			QDomElement tag = doc.createElement( "HTMLColor" );
			tag.setAttribute( "color", (*it).value() );
			tag.setAttribute( "name", (*it).name() );
			root.appendChild( tag );
		}

		QString xml = doc.toString( 4 );
		QTextStream stream( &XmlFile );
		stream << xml;
		XmlFile.close();
	}
}

void BaseDlg::modeChanged( bool mode )
{
	AdvMode = mode;
	ColorsTab->setCurrentIndex( AdvMode );
}

void BaseDlg::updateBGColor()
{
	AdvModeTab->setBGColor( BGColor );
	BasicModeTab->setBGColor( BGColor );
	SettingsTab->setBGColor( BGColor );
}

void BaseDlg::updateColors()
{
	AdvModeTab->updateColors();
	BasicModeTab->updateColors();
}

void BaseDlg::pickColor( const QStringList& groups )
{
	if( groups.size() > 0 )
	{
		qDebug( "BaseDlg::pickColor( const QStringList& groups ): list has %d members",
			groups.size() );
		pickColor( groups.at( 0 ) );
		AOColor color1 = Colors.getByName( groups.at( 0 ) );
		qDebug( "BaseDlg::pickColor( const QStringList& groups ): Color 0 is '%s' (%s)",
			qPrintable( color1.name() ), qPrintable( color1.value() ) );
		AOColor color2;
		for( int n = 1; n < groups.size(); n++ )
		{
			color2 = Colors.getByName( groups.at( n ) );
			color2.setColor( color1.value() );
			Colors.updateColor( color2 );
			qDebug( "BaseDlg::pickColor( const QStringList& groups ): Color %d is '%s' (%s)",
				n, qPrintable( color2.name() ), qPrintable( color2.value() ) );
		}
		updateColors();
	}
}

void BaseDlg::pickColor( const QString& name )
{
	AOColor color;
	if( name == "background" )
		color = BGColor;
	else
		color = Colors.getByName( name );
	QColor newColor = QColorDialog::getColor( color.color(), this );
	if( newColor.isValid() )
	{
		if( name == "background" )
		{
			BGColor.setColor( newColor );
			updateBGColor();
		}
		else
		{
			color.setColor( newColor );
			Colors.updateColor( color );
			updateColors();
		}
	}
}
